#
# Copyright (C) 2013 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
import anyconfig.globals as TT
import anyconfig.tests.common as C

import unittest


class Test_00(unittest.TestCase):
    pass

# vim:sw=4:ts=4:et:
